@@ -348,6 +348,9 @@ GdipDeleteGraphics (GpGraphics *graphics)
 		cairo_destroy (graphics->ct);
 	graphics->ct = NULL;
 
+	if (graphics->image)
+		((GpImage*) graphics->image)->graphics = NULL;
+
 	GdipFree (graphics);
 
 	return Ok;
@@ -1561,25 +1564,24 @@ GdipFillRegion (GpGraphics *graphics, GpBrush *brush, GpRegion *region)
 static int
 CalculateStringSize (GDIPCONST GpFont *gdiFont, const unsigned char *utf8, unsigned long StringDetailElements, GpStringDetailStruct *StringDetails)
 {
-	cairo_matrix_t		SavedMatrix;
-	cairo_ft_font_t		*Font;
+	cairo_font_t		*Font;
 	cairo_glyph_t		*Glyphs		= NULL;
 	GpStringDetailStruct	*CurrentDetail;
 	size_t			NumOfGlyphs;
 	float			*GlyphWidths;
 	float			TotalWidth	= 0;
 	int			i;
+	cairo_matrix_t		matrix;
 
 #ifdef DRAWSTRING_DEBUG
 	printf("CalculateStringSize(font, %s, %d, details) called\n", utf8, StringDetailElements);
 #endif
-	Font=(cairo_ft_font_t *)gdiFont->cairofnt;
+	Font=(cairo_font_t *)gdiFont->cairofnt;
 
 	/* Generate Glyhps for string utf8 */
-	cairo_matrix_copy (&SavedMatrix, (const cairo_matrix_t *)&Font->base.matrix);
-	cairo_matrix_scale (&Font->base.matrix, gdiFont->sizeInPixels, gdiFont->sizeInPixels);
-	gdpi_utf8_to_glyphs (Font, utf8, 0.0, 0.0, &Glyphs, &NumOfGlyphs);
-	cairo_matrix_copy (&Font->base.matrix, (const cairo_matrix_t *)&SavedMatrix);
+	cairo_font_current_transform(Font, &matrix);
+	cairo_matrix_scale(&matrix, gdiFont->sizeInPixels, gdiFont->sizeInPixels);
+	gdpi_utf8_to_glyphs (Font, matrix, utf8, 0.0, 0.0, &Glyphs, &NumOfGlyphs);
 
 	/* FIXME - This check and the StringDetailElements argument can be removed after verification of Glyph:WChar=1:1 */
 	if (StringDetailElements!=NumOfGlyphs) {
@@ -1600,7 +1602,7 @@ CalculateStringSize (GDIPCONST GpFont *gdiFont, const unsigned char *utf8, unsig
 		CurrentDetail++;
 	}
 
-	CurrentDetail->Width=DOUBLE_FROM_26_6(Font->face->glyph->advance.x);
+	CurrentDetail->Width=DOUBLE_FROM_26_6(cairo_ft_font_face(Font)->glyph->advance.x);
 	TotalWidth+=CurrentDetail->Width;
 	
 #ifdef DRAWSTRING_DEBUG
@@ -1683,10 +1685,10 @@ MeasureOrDrawString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int l
 	*/
 	cairo_save (graphics->ct);
 	cairo_set_font (graphics->ct, (cairo_font_t*) font->cairofnt);
-	cairo_matrix_copy (&SavedMatrix, (const cairo_matrix_t *)&font->cairofnt->base.matrix);
+	cairo_font_current_transform(font->cairofnt, &SavedMatrix);
 	cairo_scale_font (graphics->ct, font->sizeInPixels);
 	cairo_current_font_extents (graphics->ct, &FontExtent);
-	cairo_matrix_copy (&font->cairofnt->base.matrix, (const cairo_matrix_t *)&SavedMatrix);
+	cairo_font_set_transform(font->cairofnt, &SavedMatrix);
 	cairo_restore (graphics->ct);
 	LineHeight=FontExtent.ascent;
 #ifdef DRAWSTRING_DEBUG
@@ -2155,16 +2157,20 @@ MeasureOrDrawString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int l
 #ifdef DRAWSTRING_DEBUG
 			printf("Setting clipping rectangle (%d, %d %dx%d)\n", rc->X, rc->Y, rc->Width, rc->Height);
 #endif
-			cairo_init_clip (graphics->ct);
+			/* Commented following clipping lines to fix DrawString bugs */
+			/* cairo cvs seems to have fixed something which lets us */
+			/* uncomment following clipping lines */
+	 
+			/* cairo_init_clip (graphics->ct); */
 			cairo_rectangle (graphics->ct, rc->X, rc->Y, rc->Width, rc->Height);
-			cairo_clip (graphics->ct);
+			/* cairo_clip (graphics->ct); */
 			cairo_new_path (graphics->ct);
 		}
 
 		/* Setup cairo */
 		/* Save the font matrix */
 		cairo_set_font (graphics->ct, (cairo_font_t*) font->cairofnt);
-		cairo_matrix_copy (&SavedMatrix, (const cairo_matrix_t *)&font->cairofnt->base.matrix);
+		cairo_font_current_transform(font->cairofnt, &SavedMatrix);
 
 		if (brush) {
 			gdip_brush_setup (graphics, (GpBrush *)brush);
@@ -2301,13 +2307,14 @@ MeasureOrDrawString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int l
 			}
 		}
 
-		cairo_matrix_copy (&font->cairofnt->base.matrix, (const cairo_matrix_t *)&SavedMatrix);
+		cairo_font_set_transform(font->cairofnt, &SavedMatrix);
 		cairo_restore (graphics->ct);
 	}
 
 Done:
 	/* We need to remove the clip region */
-	cairo_init_clip (graphics->ct);
+	/* Following line is commented to fix the DrawString bugs */
+	/* cairo_init_clip (graphics->ct); */
 
 	/* Cleanup */
 	free (CleanString);
@@ -2606,7 +2613,7 @@ GdipSetClipPath (GpGraphics *graphics, GpPath *path, CombineMode combineMode)
 GpStatus
 GdipSetClipRegion (GpGraphics *graphics, GpRegion *region, CombineMode combineMode)
 {
-	if (!graphics || region)
+	if (!graphics || !region)
 		return InvalidParameter;
 		
 	GdipSetEmpty (graphics->clip);
